library("EML")
library("rmarkdown")

# Disclaimer: this script was written on July 4, 2016 while the EML R package is still
# under development and may not work at a later time.

# This script creates EML metadata for two tables of data from Sparkling Lake, Wisconsin.
# One contains water temperature, the other met and DO data for 2014. Both are available 
# for download at http://gleon.org/data/gleon18/dataworkshop

#first do the water temperature table
df <- read.csv("Sparkling2014wtemp.csv", header=TRUE, sep=",", quote("\""))
str(df)

#set up the data frame for metadata
rows <- ncol(df)
attributes <- data.frame(attributeName = character(rows), 
                         formatString = character(rows), 
                         unit = character(rows), 
                         numberType = character(rows), 
                         definition = character(rows), 
                         attributeDefinition = character(rows),
                         columnClasses = character(rows),
                         minimum = character(rows),
                         maximum = character(rows),
                         stringsAsFactors = FALSE)

#get some metadata from data frame
attributes$attributeName <- names(df)

attributes$columnClasses <- sapply(df, class)
#set what R thinks is integer to numeric
attributes$columnClasses[attributes$columnClasses == "integer"] <- "numeric"

edit(attributes)
write.csv(attributes, file = "sparkling2014Metadatawtemp.csv", row.names = FALSE)

#open file in excel and fill in appropriate cells

#look at standard units available for use
standardUnits <- get_unitList()
View(standardUnits$units)

#edit the metadata file in Excel

#than read it back in
attributes <- read.csv("sparkling2014Metadatawtemp.csv", header = TRUE, sep = ",", quote = "\"", as.is = TRUE)
edit(attributes)
str(attributes)


# get the column classes into a vector as required by the set_attribute function
col_classes <- attributes[,"columnClasses"]

#take out that column again
attributes$columnClasses <- NULL

#add code definitions for the flag column - format for readability
flag_wtemp <- c(D = "Sensor malfunction produced bad values set to missing",
                H = "Data suspect values outside of expected range",
                L = "Non standard routine followed")

#turn them into a data frame with three columns - attributeName, code, definition
factors <- rbind(
  data.frame(
    attributeName = "flag_wtemp",
    code = names(flag_wtemp),
    definition = unname(flag_wtemp)
  )
)
str(factors)

#with these two data frames in place we can create the attributeList element
attributeList <- set_attributes(attributes, factors, col_classes)

#physical parameter are all set for standard Microsoft csv file
physical <- set_physical("Sparkling2014wtemp.csv")

#pull to gether information for the dataTable
dataTable <- new("dataTable",
                 entityName = "Sparkling2014wtemp.csv",
                 entityDescription = "minute water temperature data in Lake Mendota",
                 physical = physical,
                 attributeList = attributeList)


#then do the met do table
#first do the water temperature table
df2 <- read.csv("Sparkling2014domet.csv", header=TRUE, sep=",", quote("\""))
str(df2)

#set up the data frame for metadata
rows2 <- ncol(df2)
attributes2 <- data.frame(attributeName = character(rows2), 
                         formatString = character(rows2), 
                         unit = character(rows2), 
                         numberType = character(rows2), 
                         definition = character(rows2), 
                         attributeDefinition = character(rows2),
                         columnClasses = character(rows2),
                         minimum = character(rows2),
                         maximum = character(rows2),
                         stringsAsFactors = FALSE)

#get some metadata from data frame
attributes2$attributeName <- names(df2)

attributes2$columnClasses <- sapply(df2, class)
#set what R thinks is integer to numeric
attributes2$columnClasses[attributes2$columnClasses == "integer"] <- "numeric"

edit(attributes2)
write.csv(attributes2, file = "sparkling2014Metadatadomet.csv", row.names = FALSE)

#open file in excel and fill in appropriate cells

#define custom unit for PAR
unitType <- data.frame(id = c("amountOfSubstanceAreaFlux", "amountOfSubstanceAreaFlux", "amountOfSubstanceAreaFlux"), dimension = c("amount", "area", "time"), power = c(0.000001, 1, 1) )
custom_units <- data.frame(id = "micromolesPerSquareMeterPerSecond", unitType = "amountOfSubstanceAreaFlux", parentSI = "molesPerMeterSquaredPerSecond", multiplierToSI = 0.000001, description = "photosynthetic active radiation")
unitsList <- set_unitList(custom_units, unitType)

#edit the metadata file in Excel

#than read it back in
attributes2 <- read.csv("sparkling2014Metadatadomet.csv", header = TRUE, sep = ",", quote = "\"", as.is = TRUE)
edit(attributes2)
str(attributes2)


# get the column classes into a vector as required by the set_attribute function
col_classes2 <- attributes2[,"columnClasses"]

#take out that column again
attributes2$columnClasses <- NULL

#add code definitions for the flag column - format for readability
flag_air_temp <- c(C = "Sensor off; missing value reported",
                H = "Data suspect values outside of expected range")
flag_rel_hum <- c(C = "Sensor off; missing value reported",
                   H = "Data suspect values outside of expected range")
flag_wind_speed_2m <- c(C = "Sensor off; missing value reported",
                   H = "Data suspect values outside of expected range")
flag_par <- c(C = "Sensor off; missing value reported",
                   H = "Data suspect values outside of expected range")
flag_opt_wtemp <- c(C = "Sensor off; missing value reported",
                   H = "Data suspect values outside of expected range")
flag_opt_do_raw <- c(C = "Sensor off; missing value reported",
                   H = "Data suspect values outside of expected range")
flag_wind_dir <- c(C = "Sensor off; missing value reported",
                   H = "Data suspect values outside of expected range")

#turn them into a data frame with three columns - attributeName, code, definition
factors2 <- rbind(
  data.frame(
    attributeName = "flag_air_temp",
    code = names(flag_air_temp),
    definition = unname(flag_air_temp)
  ),
  data.frame(
    attributeName = "flag_rel_hum",
    code = names(flag_rel_hum),
    definition = unname(flag_rel_hum)
  ),
  data.frame(
    attributeName = "flag_wind_speed_2m",
    code = names(flag_wind_speed_2m),
    definition = unname(flag_wind_speed_2m)
  ),
  data.frame(
    attributeName = "flag_par",
    code = names(flag_par),
    definition = unname(flag_par)
  ),
  data.frame(
    attributeName = "flag_opt_wtemp",
    code = names(flag_opt_wtemp),
    definition = unname(flag_opt_wtemp)
  ),
  data.frame(
    attributeName = "flag_opt_do_raw",
    code = names(flag_opt_do_raw),
    definition = unname(flag_opt_do_raw)
  ),
  data.frame(
    attributeName = "flag_wind_dir",
    code = names(flag_wind_dir),
    definition = unname(flag_wind_dir)
  )
)
str(factors)

#with these two data frames in place we can create the attributeList element
attributeList2 <- set_attributes(attributes2, factors2, col_classes2)

#physical parameter are all set for standard Microsoft csv file
physical2 <- set_physical("Sparkling2014domet.csv")

#pull to gether information for the dataTable
dataTable2 <- new("dataTable",
                 entityName = "Sparkling2014domet.csv",
                 entityDescription = "above water meteorological, under water physiologial measurements every minute",
                 physical = physical2,
                 attributeList = attributeList2)


#set the coverage elements temporal, taxonomic, geographic
begindate <- "2014-05-01"
enddate <- "2014-11-30"
geographicDescription <- "Sparkling Lake, Vilas County, Wisconsin USA"
coverage <- set_coverage(begin = begindate, end = enddate,
               #sci_names = "Sarracenia purpurea",
               geographicDescription = geographicDescription,
               west = -89.704160, east = -89.695191, 
               north = 46.015468, south = 46.002189,
               altitudeMin = 500, altitudeMaximum = 500,
               altitudeUnits = "meter")

#add the creators

R_persons <- c(as.person("John Magnuson <john.magnuson@wisc.edu>"), 
               as.person("Stephen Carpenter <steve.carpenter@wisc.edu>"),
               as.person("Emily Stanley <ehstanley@wisc.edu>"),
               as.person("NTL Lead-PI <leadpi@lter.limnology.wisc.edu>"))

creators <- as(R_persons, "creator")

#add the publisher
NTL_address <- new("address",
                  deliveryPoint = "Center for Limnology 680 North Park Str.",
                  city = "Madison",
                  administrativeArea = "WI",
                  postalCode = "53706",
                  country = "USA")

publisher <- new("publisher",
                 organizationName = "North Temperate Lakes LTER",
                 address = NTL_address)

#add the contact
NTL_contact <- new("individualName",
                   givenName = "NTL",
                   surName = "Information-Manager")

contact <- 
  new("contact",
      individualName = NTL_contact,
      electronicMail = "infomgr@lter.limnology.wisc.edu",
      address = NTL_address,
      organizationName = "North Temperate Lakes LTER",
      phone = "608-890-3446")

#add keywords
keywordSet <-
  c(new("keywordSet",
        keywordThesaurus = "LTER controlled vocabulary",
        keyword = c("meteorology",
                    "water tempterature",
                    "air temperature",
                    "dissolved oxygen",
                    "relative humidity",
                    "wind direction",
                    "wind speed")),
    new("keywordSet",
        keywordThesaurus = "NTL Keywords",
        keyword =  c("instrumented buoy", 
                     "North Temperate Lakes - LTER", 
                     "NTL LTER",
                     "photosynthetically-active radiation",
                     "PAR")))

pubDate <- "2016"

title <- "North Temperate Lakes LTER: High Frequency Water Temperature, Meteoroloigal and Lake Metabolism Data - Sparkling Lake Buoy 2014"

abstract <- as(set_TextType("SparklingAbstract.docx"), "abstract")
intellectualRights <- as(set_TextType("intellectualRights.docx"), "intellectualRights")
methods <- set_methods("SparklingMethods.docx")


#put it all together
dataset <- new("dataset",
               title = title,
               creator = creators,
               pubDate = pubDate,
               intellectualRights = intellectualRights,
               abstract = abstract,
               #associatedParty = associatedParty,
               keywordSet = keywordSet,
               coverage = coverage,
               contact = contact,
               methods = methods,
               dataTable = c(dataTable, dataTable2))

eml <- new("eml",
           packageId = "knb-lter-ntl.10100.1",
           system = "knb",
           dataset = dataset,
           additionalMetadata = as(unitsList, "additionalMetadata"))

eml_validate(eml)
write_eml(eml, "Sparkling2014.xml")
